<?php

if( ! class_exists('Ph_Bookings_Send_Email_Notification') ) {
	class Ph_Bookings_Send_Email_Notification {

		public function __construct() {
			add_action( 'ph_bookings_notification_cron', array( $this, 'ph_bookings_notification_cron_run') );
			add_filter( 'cron_schedules', array($this,'ph_bookings_notification_cron' ));
			$this->settings 		= Ph_Booking_Reminder_Common::get_booking_reminder_settings();
			$this->enabled			= ! empty($this->settings['enable']) ? $this->settings['enable'] : false;
			$this->email_subject	= ! empty($this->settings['email_subject']) ? $this->settings['email_subject'] : __( 'Bookings Reminders', 'ph-bookings-reminder-notifications');
			$this->email_content	= ! empty($this->settings['email_content']) ? $this->settings['email_content'] : Ph_Booking_Reminder_Common::get_default_email_content();
			$this->notification_time			= ! empty($this->settings['notification_time']) ? $this->settings['notification_time'] : 60;
		}

		public function ph_bookings_notification_cron( $schedules ) {
			$import_interval=1;
		    $schedules['booking_reminder_interval'] = array(
		            'interval'  => (int) $import_interval * 60 ,
		            'display'   => sprintf(__('Every %d minutes', 'bookings-and-appointments-for-woocommerce'), (int) $import_interval)
		    );
		    return $schedules;
		}
		/**
		 * ph_bookings_notification_cron_run
		 */
		public function ph_bookings_notification_cron_run(){
			$this->site_title = wp_specialchars_decode( get_option('blogname'), ENT_QUOTES );
			// Get Next Day Date according to site time
			$date_from = date_create();
			date_timestamp_set( $date_from, current_time('timestamp') );

			$date_to = date_create();
			date_timestamp_set( $date_to, current_time('timestamp') );
			$date_to->modify("5 days");

			$filters = array(
				'ph_booking_status' => null,
				'ph_filter_from'	=> $date_from->format('Y-m-d'),
				'ph_filter_to'		=> $date_to->format('Y-m-d')
			);

			$this->bookings = $this->get_bookings( $filters );
			$this->bookings = $this->remove_previously_send_bookings( $this->bookings );
			$this->send_email( $this->bookings );
		}

		/**
		 * Remove the Bookings which already send.
		 * @param array $bookings Bookings
		 * @param object $date_to_select
		 */
		public function remove_previously_send_bookings( $bookings ) {
			
			foreach( $bookings as $key => $booking ) {

				// Don't include Cancelled Bookings
				if(  !empty($booking['ReminderEmailStatus']) ) {
					$ReminderEmailStatus 			= unserialize($booking['ReminderEmailStatus']);
					if($ReminderEmailStatus)
					{		
						unset($bookings[$key]);
						continue;
					}
				}
			}
			// error_log('removed');
			// error_log(print_r($bookings,1));
			return $bookings;
		}

		/**
		 * Send Email for the Bookings
		 * @param array $bookings Array of Bookings for which email need to be sent
		 */
		public function send_email( $bookings = array() ){
			$header = array(
				"Content-Type: text/html; charset=UTF-8"
			);
			$from_name		= get_option( 'woocommerce_email_from_name');
			$from_address	= get_option( 'woocommerce_email_from_address' );
			$header[]		= "From : ".wp_specialchars_decode( esc_html($from_name), ENT_QUOTES )." <$from_address>";
			$subject		= $this->get_email_subject();
			foreach( $bookings as $booking ) {
				$email_status=wp_mail( $booking['billing_email'], $subject, $this->get_email_content($booking), $header );
				// $logger = wc_get_logger();
				// $context = array( 'source' => 'reminder email addon email status' );
				// $logger->debug( json_encode($email_status), $context );
				if($email_status)
				{
					$order_item_id=$booking['ID'];
					wc_add_order_item_meta( $order_item_id, 'ReminderEmailStatus', array(true));
				}
			}
		}

		/**
		 * Get Email Subject.
		 * @return string Email Subject.
		 */
		private function get_email_subject() {
			return $this->email_subject;
		}

		/**
		 * Get Email Content
		 * @param array $booking Booking
		 * @return string Email Content / Message.
		 */
		public function get_email_content( $booking ) {
			$email_content = str_replace( array( PHP_EOL, '[CUSTOMER_NAME]', '[SITE_NAME]'), array( "<br>", $booking['bookedby'], $this->site_title), $this->email_content );
			// $email_content="testing";
			$email_content =str_replace('[BOOKING_DETAILS]', $this->get_product_info_as_table($booking), $email_content);
			return $email_content;
		}

		public function get_bookings( $filters ) {
			return $this->ph_get_bookings($filters);
		}

		/**
		 * Get Bookings.
		 * @param array $filters Bookings Filters
		 */
		private function ph_get_bookings( $filters ){
			global $wpdb;
			$query = "SELECT oitems.order_id, oitems.order_item_id,tr.object_id product_id,ometa.customer_name, ometa.billing_email, imeta.BookingStatus, imeta.BookFrom, imeta.BookTo, imeta.IntervalDetails,imeta.ReminderEmailStatus
			FROM {$wpdb->prefix}posts
			INNER JOIN (
					SELECT
					post_id,
					MAX(CASE WHEN meta_key = '_billing_email' THEN meta_value ELSE '' END) AS billing_email,
					MAX(CASE WHEN meta_key = '_billing_first_name' THEN meta_value ELSE '' END) AS customer_name
					FROM {$wpdb->prefix}postmeta
					GROUP BY post_id
			) as ometa on ometa.post_id = {$wpdb->prefix}posts.ID
			INNER JOIN {$wpdb->prefix}woocommerce_order_items oitems on oitems.order_id = {$wpdb->prefix}posts.ID
			INNER JOIN (
					SELECT
					order_item_id,
					MAX(CASE WHEN meta_key = '_product_id' THEN meta_value ELSE '' END) AS ProductId,
					MAX(CASE WHEN meta_key = 'booking_status' THEN meta_value ELSE '' END) AS BookingStatus,
					MAX(CASE WHEN meta_key = 'From' THEN meta_value ELSE '' END) AS BookFrom,
					MAX(CASE WHEN meta_key = 'To' THEN meta_value ELSE '' END) AS BookTo,
					MAX(CASE WHEN meta_key = '_phive_booking_product_interval_details' THEN meta_value ELSE '' END) AS IntervalDetails,
					MAX(CASE WHEN meta_key = 'ReminderEmailStatus' THEN meta_value ELSE '' END) AS ReminderEmailStatus
					FROM {$wpdb->prefix}woocommerce_order_itemmeta
					GROUP BY order_item_id
			) as imeta on  imeta.order_item_id = oitems.order_item_id
			INNER JOIN {$wpdb->prefix}term_relationships AS tr ON tr.object_id = imeta.ProductId
			INNER JOIN {$wpdb->prefix}term_taxonomy AS tt ON tr.term_taxonomy_id = tt.term_taxonomy_id
			INNER JOIN {$wpdb->prefix}terms AS t ON t.term_id = tt.term_id
			WHERE {$wpdb->prefix}posts.post_type IN ('shop_order', 'shop_order_refund')
			AND (
				{$wpdb->prefix}posts.post_status = 'wc-pending'
				OR {$wpdb->prefix}posts.post_status = 'wc-processing'
				OR {$wpdb->prefix}posts.post_status = 'wc-on-hold'
				OR {$wpdb->prefix}posts.post_status = 'wc-completed'
			)
			AND tt.taxonomy IN ('product_type')
			AND t.slug = 'phive_booking'";
			
			$sub_query = "
				IF( LENGTH( SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:REGEXP ".'"[7-16]"'.":".'"'."', -1), '".'"'."', -2),'".'"'."',1) ) = 7,
					CONCAT( SUBSTRING_INDEX(SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:7:".'"'."', -1), '".'"'."', -2),'".'"'."',1),'-01' ),
				SUBSTRING_INDEX( SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookFrom, 'i:0;s:REGEXP ".'"[10-16]"'.":".'"'."', -1), '".'"'."', -2),'".'"'."',1) )";
	
			if( !empty($filters['ph_booking_status']) ){
				$len = strlen($filters['ph_booking_status']);
				$query .= " AND (SUBSTRING_INDEX(SUBSTRING_INDEX(imeta.BookingStatus, 'i:0;s:
				".$len.":".'"'."', -1), '".'"'."', 1) = '".$filters['ph_booking_status']."' OR imeta.BookingStatus = '".$filters['ph_booking_status']."' ) ";
			}
			if( !empty($filters['ph_filter_product_ids']) ){
				$query .= " AND imeta.ProductId = '".$filters['ph_filter_product_ids']."'";
			}
			if( !empty($filters['ph_filter_from']) ){
	
				$query .= " AND ( DATE(".$sub_query.")  >= '".$filters['ph_filter_from']."'";
				
				$query .= " OR DATE(imeta.BookFrom) >= '".$filters['ph_filter_from']."')";
				
			}
			if( !empty($filters['ph_filter_to']) ){
				$query .= " AND (DATE(".$sub_query.") <= '".$filters['ph_filter_to']."'";
				$query .= " OR DATE(imeta.BookFrom) <= '".$filters['ph_filter_to']."')";
				
			}
			
			
			$results = $wpdb->get_results( $query );
			$bookings = array();
			
			// $logger = wc_get_logger();
			// $context = array( 'source' => 'reminder email addon' );
			// $logger->debug( json_encode($results), $context );
			
			$date_from = date_create();
			date_timestamp_set( $date_from, current_time('timestamp') );
			$email_interval=$this->notification_time;
			$date1=date('Y-m-d H:i',strtotime("+$email_interval minutes",strtotime($date_from->format('Y-m-d H:i'))));
			$email_interval=$email_interval-5;
			$date2=date('Y-m-d H:i',strtotime("+$email_interval minutes",strtotime($date_from->format('Y-m-d H:i'))));
			foreach ($results as $key => $result) {
				$IntervalDetails	= unserialize($result->IntervalDetails);
				$BookFrom 			= unserialize($result->BookFrom);
				if(!is_array($BookFrom) && !$BookFrom )
				{
					$BookFrom=$result->BookFrom;
				}
				else
				{
					$BookFrom=$BookFrom[0];
				}
				$booking_from=date('Y-m-d H:i',strtotime($BookFrom));
				
				// $logger->debug( json_encode($result->BookFrom), $context );
				// $logger->debug( json_encode($BookFrom), $context );
				// $logger->debug( json_encode($booking_from), $context );
				// $logger->debug( json_encode($date1), $context );
				// $logger->debug( json_encode($date2), $context );
				// $logger->debug( json_encode(strtotime($booking_from)), $context );
				// $logger->debug( json_encode(strtotime($date1)), $context );
				// $logger->debug( json_encode(strtotime($date2)), $context );
				// $logger->debug( json_encode(strtotime($booking_from) > strtotime($date1)), $context );
				// $logger->debug( json_encode(strtotime($booking_from) < strtotime($date2)), $context );
				if(strtotime($booking_from) > strtotime($date1) || strtotime($booking_from) < strtotime($date2)  )
				{
					continue;
				}
				$bookings[] = array(
					'ID' 			=> $result->order_item_id,
					'order_id' 		=> $result->order_id,
					'product_id' 	=> $result->product_id,
					'start' 		=> $BookFrom,
					'bookedby' 		=> $result->customer_name,
					'billing_email'	=> $result->billing_email,
					'booking_status'=> ph_maybe_unserialize($result->BookingStatus),
					'ReminderEmailStatus'=>$result->ReminderEmailStatus
				);
			}
			// $logger = wc_get_logger();
			// $context = array( 'source' => 'reminder email addon' );
			// $logger->debug( json_encode($bookings), $context );
			// error_log($date_from->format('Y-m-d H:i'));
			// error_log(print_r($bookings,1));
			return $bookings;
		}

		/**
		 * Get bookings product info as table.
		 */
		private function get_product_info_as_table($booking) {
			$order_item_id=$booking['ID'];
			$order_id=$booking['order_id'];
			$content = null;
			$order = wc_get_order($order_id);
			if( $order instanceof WC_Order ) {
				$order_items = $order->get_items();
				$table_td_style = "style='border: 1px solid #dddddd; padding:10px;font-size:17px;'";
				$table_td_title_style = "style='border: 1px solid #dddddd; padding:10px;font-size:20px;'";
				if( ! empty($order_items) ) {
					$content .= "<table  style='border-collapse:collapse; width:100%;'";
						$content .= "<tr>
										<td $table_td_title_style>".__( 'Product', 'bookings-and-appointments-for-woocommerce')."</th>
										<td $table_td_title_style>".__( 'Price', 'bookings-and-appointments-for-woocommerce')."</th>
									</tr>";
						foreach( $order_items as $order_item ) {
							$product 	= $order_item->get_product();
							$item_id 	= $order_item->get_id();
							if( empty($product) || $product->get_type() !='phive_booking' || $item_id!=$order_item_id){
								continue;
							}
							$content .= "<tr>
											<td $table_td_style>".$order_item->get_name().$this->get_order_item_meta_data($order_item)."</td>".
											"<td $table_td_style>".wp_kses_post( $order->get_formatted_line_subtotal( $order_item ))."</td>
										</tr>";
						}
					$content .= "</table>";
				}
			}
			return $content;
		}

		/**
		 * Get Meta data of Line Items that needs to be sent in email.
		 * @return string in the form of Unordered list
		 */
		private function get_order_item_meta_data( $order_item ) {
			$content = null;
			$meta_datas = $order_item->get_meta_data();
			$key_filter=apply_filters('ph_bookings_order_meta_key_filters',array('confirmed','canceled'));
			foreach( $meta_datas as $meta_data ) {
				$meta_data = $meta_data->get_data();
				if( ! empty($meta_data['value']) && ! is_array($meta_data['value']) ) {
					if( in_array($meta_data['key'], $key_filter) )	continue;
					$content .= "<li>".__($meta_data['key'], 'bookings-and-appointments-for-woocommerce').": ".__($meta_data['value'], 'bookings-and-appointments-for-woocommerce')."</li>";
				}
			}

			if( ! empty($content) ) {
				$content = "<ul style='padding:10px;font-size:12px;'>".$content."</ul>";
			}
			return $content;
		}

	}
	new Ph_Bookings_Send_Email_Notification();
}